@@ -1,4 +1,5 @@
 import importlib
+import inspect
 import times
 from uuid import uuid4
 from cPickle import loads, dumps, UnpicklingError
@@ -6,6 +7,11 @@ from .connections import get_current_connection
 from .exceptions import UnpickleError, NoSuchJobError
 
 
+JOB_ATTRS = set(['origin', '_func_name', 'ended_at', 'description', '_args',
+                 'created_at', 'enqueued_at', 'connection', '_result',
+                 'timeout', '_kwargs', 'exc_info', '_id', 'data', '_instance'])
+
+
 def unpickle(pickled_string):
     """Unpickles a string, but raises a unified UnpickleError in case anything
     fails.
@@ -50,7 +56,13 @@ class Job(object):
         """
         connection = kwargs.pop('connection', None)
         job = cls(connection=connection)
-        job._func_name = '%s.%s' % (func.__module__, func.__name__)
+        if inspect.ismethod(func):
+            job._instance = func.im_self
+            job._func_name = func.__name__
+        elif inspect.isfunction(func):
+            job._func_name = '%s.%s' % (func.__module__, func.__name__)
+        else:  # we expect a string
+            job._func_name = func
         job._args = args
         job._kwargs = kwargs
         job.description = job.get_call_string()
@@ -66,10 +78,17 @@ class Job(object):
         if func_name is None:
             return None
 
+        if self.instance:
+            return getattr(self.instance, func_name)
+
         module_name, func_name = func_name.rsplit('.', 1)
         module = importlib.import_module(module_name)
         return getattr(module, func_name)
 
+    @property
+    def instance(self):
+        return self._instance
+
     @property
     def args(self):
         return self._args
@@ -100,6 +119,7 @@ class Job(object):
         self._id = id
         self.created_at = times.now()
         self._func_name = None
+        self._instance = None
         self._args = None
         self._kwargs = None
         self.description = None
@@ -136,12 +156,11 @@ class Job(object):
         """The Redis key that is used to store job hash under."""
         return self.key_for(self.id)
 
-
     @property  # noqa
     def job_tuple(self):
         """Returns the job tuple that encodes the actual function call that
         this job represents."""
-        return (self.func_name, self.args, self.kwargs)
+        return (self.func_name, self.instance, self.args, self.kwargs)
 
     @property
     def return_value(self):
@@ -176,12 +195,8 @@ class Job(object):
         Will raise a NoSuchJobError if no corresponding Redis key exists.
         """
         key = self.key
-        properties = ['data', 'created_at', 'origin', 'description',
-                'enqueued_at', 'ended_at', 'result', 'exc_info', 'timeout']
-        data, created_at, origin, description, \
-                enqueued_at, ended_at, result, \
-                exc_info, timeout = self.connection.hmget(key, properties)
-        if data is None:
+        obj = self.connection.hgetall(key)
+        if not obj:
             raise NoSuchJobError('No such job: %s' % (key,))
 
         def to_date(date_str):
@@ -190,18 +205,20 @@ class Job(object):
             else:
                 return times.to_universal(date_str)
 
-        self._func_name, self._args, self._kwargs = unpickle(data)
-        self.created_at = to_date(created_at)
-        self.origin = origin
-        self.description = description
-        self.enqueued_at = to_date(enqueued_at)
-        self.ended_at = to_date(ended_at)
-        self._result = result
-        self.exc_info = exc_info
-        if timeout is None:
-            self.timeout = None
-        else:
-            self.timeout = int(timeout)
+        self._func_name, self._instance, self._args, self._kwargs = unpickle(obj.get('data'))  # noqa
+        self.created_at = to_date(obj.get('created_at'))
+        self.origin = obj.get('origin')
+        self.description = obj.get('description')
+        self.enqueued_at = to_date(obj.get('enqueued_at'))
+        self.ended_at = to_date(obj.get('ended_at'))
+        self._result = obj.get('result')
+        self.exc_info = obj.get('exc_info')
+        self.timeout = int(obj.get('timeout')) if obj.get('timeout') else None
+
+        # Overwrite job's additional attrs (those not in JOB_ATTRS), if any
+        additional_attrs = set(obj.keys()).difference(JOB_ATTRS)
+        for attr in additional_attrs:
+            setattr(self, attr, obj[attr])
 
     def save(self):
         """Persists the current job instance to its corresponding Redis key."""
@@ -226,7 +243,18 @@ class Job(object):
             obj['exc_info'] = self.exc_info
         if self.timeout is not None:
             obj['timeout'] = self.timeout
+        """
+        Store additional attributes from job instance into Redis. This is done
+        so that third party libraries using RQ can store additional data
+        directly on ``Job`` instances. For example:
 
+        job = Job.create(func)
+        job.foo = 'bar'
+        job.save() # Will persist the 'foo' attribute
+        """
+        additional_attrs = set(self.__dict__.keys()).difference(JOB_ATTRS)
+        for attr in additional_attrs:
+            obj[attr] = getattr(self, attr)
         self.connection.hmset(key, obj)
 
     def cancel(self):
