@@ -74,7 +74,7 @@ class Queue(object):
         """Returns a list of all (valid) jobs in the queue."""
         def safe_fetch(job_id):
             try:
-                job = Job.fetch(job_id)
+                job = Job.fetch(job_id, connection=self.connection)
             except NoSuchJobError:
                 return None
             except UnpickleError:
@@ -112,17 +112,18 @@ class Queue(object):
         it.
 
         Expects the function to call, along with the arguments and keyword
-        arguments.
+        arguments.  May be a fully qualified string of the function instance,
+        in which case the function must be meaningful to the worker.
 
         The special keyword `timeout` is reserved for `enqueue()` itself and
         it won't be passed to the actual job function.
         """
-        if f.__module__ == '__main__':
+        if not isinstance(f, basestring) and f.__module__ == '__main__':
             raise ValueError(
                     'Functions from the __main__ module cannot be processed '
                     'by workers.')
 
-        timeout = kwargs.pop('timeout', None)
+        timeout = kwargs.pop('timeout', self._default_timeout)
         job = Job.create(f, *args, connection=self.connection, **kwargs)
         return self.enqueue_job(job, timeout=timeout)
 
@@ -263,7 +264,7 @@ class FailedQueue(Queue):
         """
         job.ended_at = times.now()
         job.exc_info = exc_info
-        return self.enqueue_job(job, set_meta_data=False)
+        return self.enqueue_job(job, timeout=job.timeout, set_meta_data=False)
 
     def requeue(self, job_id):
         """Requeues the job with the given job ID."""
@@ -280,4 +281,4 @@ class FailedQueue(Queue):
 
         job.exc_info = None
         q = Queue(job.origin, connection=self.connection)
-        q.enqueue_job(job)
+        q.enqueue_job(job, timeout=job.timeout)
